# -*- coding: utf-8 -*-
# Generated by Django 1.9.8 on 2016-08-18 09:39
from __future__ import unicode_literals

from django.db import migrations, models
from django.conf import settings
import uuid
import os



class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='SoundFile',
            fields=[
                ('uuid', models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False)),
                ('file', models.FileField(upload_to=os.path.join(settings.BASE_DIR, 'generated'))),
                ('text', models.TextField()),
                ('command', models.CharField(blank=True, max_length=255)),
                ('type', models.CharField(choices=[('wav', 'wav')], max_length=5)),
                ('dc', models.DateTimeField(auto_now_add=True)),
            ],
        ),
    ]



# Create your views here.
import subprocess
import tempfile
import os
import uuid

from django.http import Http404, JsonResponse
from django.http import HttpResponse
from django.core.files import File
from django.shortcuts import redirect
from rest_framework.response import Response
from rest_framework import status
from rest_framework.decorators import api_view

from . import models
from . import serializers


@api_view(['post'])
def generate(request):
    """
    ---
    parameters:
    - name: text
      description: Text for generation
      required: true
      paramType: form
    """
    serializer = serializers.Generation(data=request.data)
    serializer.is_valid(raise_exception=True)
    text = serializer.validated_data['text']

    tmp_dir = '/tmp/voice/'
    if not os.path.exists(tmp_dir):
        os.makedirs(tmp_dir)

    file_uuid = str(uuid.uuid4())
    tmp_path = '%s/%s.wav' % (tmp_dir, file_uuid)

    command = 'echo %s | RHVoice-test -p elena -o %s' % (text, tmp_path)
    status_code, output = subprocess.getstatusoutput(command)
    if status_code != 0:
        return Response(
            {'error': 'File generation error', 'output': output},
            status=status.HTTP_500_INTERNAL_SERVER_ERROR
        )

    with open(tmp_path, 'rb') as tmp_file:
        models.SoundFile.objects.create(
            uuid=file_uuid,
            text=text,
            command=command,
            file=File(tmp_file),
            type='wav',
        )

    os.unlink(tmp_path)
    return JsonResponse({'status': status_code, 'uuid': file_uuid})


@api_view()
def get_file(request):
    """
    ---
    parameters:
    - name: uuid
      description: uuid of generated file
      required: true
      paramType: query
    """
    serializer = serializers.GetFile(data=request.query_params)
    serializer.is_valid(raise_exception=True)

    sound_file = models.SoundFile.objects.get(uuid=serializer.validated_data['uuid'])

    response = HttpResponse()
    response.write(sound_file.file.read())

    response['Content-Type'] = 'audio/vnd.wave'
    response['Content-Length'] = sound_file.file.size
    response['Content-Disposition'] = 'attachment; filename=%s' % sound_file.file.name.split('/')[-1]

    return response



from django.conf.urls import url, include
from . import views

urlpatterns = [
    url(r'^docs/', include('rest_framework_swagger.urls', namespace='docs')),
    url(r'^generate/$', views.generate, name='generate'),
    url(r'^get_file/$', views.get_file, name='get_file'),
]
